#!/usr/bin/python2.7 -B
# -*- coding: UTF-8 -*-
from Tkinter import *
import sys
import re
import os

class UserPass:

    BADUSER = re.compile(r'[^\x2d\x30-\x39\x41-\x5A\x5F\x61-\x7A]')
    BADPASS = re.compile(r'[\x00-\x1F\x7F-\xFF]')

    def __init__(self, window):
        self.username = ""
        self.password = ""

        frame = Frame(window, borderwidth=2, relief=RAISED)
	frame.grid()

        Label(frame, text="University of Helsinki\nUser creation via Eduroam").grid(row=0, column=0, columnspan=6, padx=16, pady=8)

        self.uservar = StringVar()
        self.uservar.trace_variable('w', self.validate)
        self.passvar = StringVar()
        self.passvar.trace_variable('w', self.validate)

        Label(frame, text=" ").grid(row=3, column=0, ipadx=4, ipady=4)
        Label(frame, text=" ").grid(row=3, column=5, ipadx=4, ipady=4)
        Label(frame, text=" ").grid(row=4, column=0, ipadx=4, ipady=4)
        Label(frame, text=" ").grid(row=4, column=5, ipadx=4, ipady=4)

        Label(frame, text="Username: ", anchor=W).grid(row=3, column=1, pady=4, padx=4, sticky="E")
        Label(frame, text="Password: ", anchor=W).grid(row=4, column=1, pady=4, padx=4, sticky="E")

        self.userentry = Entry(frame, width=16, textvar=self.uservar, exportselection=False)
        self.userentry.grid(row=3, column=2, columnspan=2, sticky=E+W)
        self.userentry.bind('<Return>', func=lambda event: self.passfocus(event))

        self.usermark = Label(frame, text=" ", fg="#c00")
        self.usermark.grid(row=3, column=4, sticky=W)

        self.passentry = Entry(frame, width=16, textvar=self.passvar, show=u'☢', exportselection=False)
        self.passentry.grid(row=4, column=2, columnspan=2, sticky=E+W)
        self.passentry.bind('<Return>', func=lambda event: self.submit(event))

        self.passmark = Label(frame, text=" ", fg="#c00")
        self.passmark.grid(row=4, column=4, sticky=W)

        self.okbutton = Button(frame, text="Ok", command=lambda: self.submit(), state=DISABLED)
        self.okbutton.bind('<Return>', func=self.submit)
        self.okbutton.grid(row=6, column=2, sticky=E)

        self.cancelbutton = Button(frame, text="Cancel", command=lambda: self.cancel())
        self.cancelbutton.bind('<Return>', func=self.cancel)
        self.cancelbutton.grid(row=6, column=3, sticky=E)

        Label(frame, text=" ").grid(row=6, column=0, ipady=8)
        Label(frame, text=" ").grid(row=6, column=5, ipady=8)

	self.quit = lambda: window.quit()

    def validate(self, *args, **kwargs):
        user_ok = self.validuser(self.uservar.get())
        pass_ok = self.validpass(self.passvar.get())
        if user_ok and pass_ok:
            self.okbutton.configure(state=NORMAL)
        else:
            self.okbutton.configure(state=DISABLED)

    def validuser(self, username):
        if len(username) < 1:
            self.usermark.configure(text=" ")
            return False
        elif self.BADUSER.search(username):
            self.usermark.configure(text="!")
            return False
        else:
            self.usermark.configure(text=" ")
            return True

    def validpass(self, password):
        if len(password) < 1:
            self.passmark.configure(text=" ")
            return False
        elif self.BADPASS.search(password):
            self.passmark.configure(text="!")
            return False
        else:
            self.passmark.configure(text=" ")
            return True

    def userfocus(self, event=None):
        self.userentry.focus_force()

    def passfocus(self, event=None):
        self.passentry.focus_force()

    def cancel(self, event=None):
        self.username = ''
        self.password = ''
        self.quit()

    def submit(self, event=None):
        username = self.uservar.get()
        password = self.passvar.get()
        userok = self.validuser(username)
        self.validpass(password)
        if userok and len(password) > 0:
            self.username = username
            self.password = password
            self.quit()
        else:
            self.username = ''
            self.password = ''


if __name__ == '__main__':
    window = Tk()

    window.wm_title(u'University of Helsinki')
    window.wm_attributes('-type', 'splash')
    window.wm_attributes('-topmost', 'true')
    window.tk_setPalette('#cde')

    dialog = UserPass(window)
    window.update()
    window.update_idletasks()
    window.update()
    dialog.userfocus()
    window.update()
    window.update_idletasks()
    window.update()

    maxw = window.winfo_vrootwidth()
    maxh = window.winfo_vrootheight()
    w = min(window.winfo_width(), maxw)
    h = min(window.winfo_height(), maxh)
    window.wm_geometry("%dx%d+%d+%d" % (w, h, (maxw-w)/2, (maxh-h)/2))

    window.mainloop()
    window.destroy()

    User = dialog.username
    Pass = dialog.password
    del dialog
    del window

    if len(User) > 0 and len(Pass) > 0:
        with os.fdopen(6, 'w') as handle:
            handle.write('%s\n%s\n' % (User, Pass))
        sys.exit(0)
    else:
        sys.exit(1)
